﻿using Microsoft.VisualBasic.FileIO;
using ProjectPolyclinic.Entities;
using ProjectPolyclinic.Entities.Enums;
using ProjectPolyclinic.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPolyclinic.Forms
{
    public partial class FormMedication : Form
    {
        private readonly IMedicationRepository _medicationRepository;
        private int? _medicationId;

        public int Id
        {
            set
            {
                try
                {
                    var medication = _medicationRepository.ReadMedicationById(value);
                    if (medication == null)
                    {
                        throw new
                        InvalidDataException(nameof(medication));
                    }
                    foreach (MedicinesType elem in
                    Enum.GetValues(typeof(MedicinesType)))
                    {
                        if ((elem & medication.MedicinesType) != 0)
                        {
                            checkedListBoxMedicinesType.SetItemChecked(checkedListBoxMedicinesType.Items.IndexOf(
                            elem), true);
                        }
                    }
                    textBoxDosing.Text = medication.Dosing;
                    richTextBoxDescription.Text = medication.Description;
                    _medicationId = value;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Ошибка при полученииданных", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;

                }
            }
        }
        public FormMedication(IMedicationRepository medicationRepository)
        {
            InitializeComponent();
            _medicationRepository = medicationRepository ?? throw new ArgumentNullException(nameof(medicationRepository));
            foreach (var elem in Enum.GetValues(typeof(MedicinesType)))
            {
                checkedListBoxMedicinesType.Items.Add(elem);
            }
        }
        private void ButtonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxDosing.Text) ||
                string.IsNullOrWhiteSpace(richTextBoxDescription.Text) ||
                checkedListBoxMedicinesType.CheckedItems.Count == 0)
                {
                    throw new Exception("Имеются незаполненные поля");
                }
                if (_medicationId.HasValue)
                {
                    _medicationRepository.UpdateMedication(CreateMedication(_medicationId.Value));
                }
                else
                {
                    _medicationRepository.CreateMedication(CreateMedication(0));
                }
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
        private void ButtonCancel_Click(object sender, EventArgs e) => Close();
        private Medication CreateMedication(int id)
        {
            MedicinesType medicinesType = MedicinesType.None;
            foreach (var elem in checkedListBoxMedicinesType.CheckedItems)
            {
                medicinesType |= (MedicinesType)elem;
            }
            return Medication.CreateEntity(id, medicinesType, textBoxDosing.Text,
            richTextBoxDescription.Text);
        }
    }



}

