﻿using ProjectPolyclinic.Entities;
using ProjectPolyclinic.Entities.Enums;
using ProjectPolyclinic.Repositories;
using ProjectPolyclinic.Repositories.Implementations;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectPolyclinic.Forms
{
    public partial class FormVisiting : Form
    {
        private readonly IVisitingRepository _visitingRepository;
        public FormVisiting(IVisitingRepository visitingRepository, IDoctorRepository doctorRepository, IPatientRepository patientRepository, IMedicationRepository medicationRepository)
        {
            InitializeComponent();
            _visitingRepository = visitingRepository ?? throw new ArgumentNullException(nameof(visitingRepository));
            comboBoxDoctor.DataSource =
            doctorRepository.ReadDoctors();
            comboBoxDoctor.DisplayMember = "FullName";
            comboBoxDoctor.ValueMember = "Id";
            comboBoxPatient.DataSource = patientRepository.ReadPatients();
            comboBoxPatient.DisplayMember = "FullName";
            comboBoxPatient.ValueMember = "Id";
            foreach (var elem in Enum.GetValues(typeof(DiagnosisName)))
            {
                checkedListBoxDiagnosisName.Items.Add(elem);
            }
            ColumnMedicines.DataSource = medicationRepository.ReadMedicines();
            ColumnMedicines.DisplayMember = "MedicinesType";
            ColumnMedicines.ValueMember = "Id";
            
        }

        private void ButtonSave_Click(object sender, EventArgs e)
        {
            try
            {
                if (comboBoxDoctor.SelectedIndex < 0 ||
                comboBoxPatient.SelectedIndex < 0 ||
                checkedListBoxDiagnosisName.CheckedItems.Count == 0)
                {
                    throw new Exception("Имеются незаполненные поля");
                }
                DiagnosisName diagnosisName = DiagnosisName.None;
                foreach (var elem in checkedListBoxDiagnosisName.CheckedItems)
                {
                    diagnosisName |= (DiagnosisName)elem;
                }
                _visitingRepository.CreateVisiting(Visiting.CreateOperation(
                0,
                (int)comboBoxDoctor.SelectedValue!,
                (int)comboBoxPatient.SelectedValue!,
                diagnosisName,
                dateTimePickerVisiting.Value, CreateListMedicines_VisitingFromDataGrid()));
                Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при сохранении",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }



        }

        private void ButtonCancel_Click(object sender, EventArgs e) => Close();

        private List<Medicines_Visiting> CreateListMedicines_VisitingFromDataGrid()
        {
            var list = new List<Medicines_Visiting>();
            foreach (DataGridViewRow row in dataGridViewMedicines.Rows)
            {
                if (row.Cells["ColumnMedicines"].Value == null ||
                row.Cells["ColumnCount"].Value == null)
                {
                    continue;
                }
               
                list.Add(Medicines_Visiting.CreateElement(Convert.ToInt32(row.Cells["ColumnMedicines"].Value),
                0,
                Convert.ToInt32(row.Cells["ColumnCount"].Value)));
            }
            return list;
        }

       
    }
}
