﻿using ProjectPolyclinic.Reports;
using ProjectPolyclinic.Repositories;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Unity;

namespace ProjectPolyclinic.Forms
{
    public partial class FormVisitingReport : Form
    {
        private readonly IUnityContainer _container;

        public FormVisitingReport(IUnityContainer container, IDoctorRepository doctorRepository)
        {
            InitializeComponent();
            _container = container ?? throw new ArgumentNullException(nameof(container));
            comboBoxDoctor.DataSource = doctorRepository.ReadDoctors();
            comboBoxDoctor.DisplayMember = "FullName";
            comboBoxDoctor.ValueMember = "Id";

        }

        private void ButtonSelectFilePath_Click(object sender, EventArgs e)
        {
            var sfd = new SaveFileDialog()
            {
                Filter = "Excel Files | *.xlsx"
            };
            if (sfd.ShowDialog() != DialogResult.OK)
            {
                return;
            }
            textBoxFilePath.Text = sfd.FileName;
        }

        private void ButtonMakeReport_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(textBoxFilePath.Text))
                {
                    throw new Exception("Отсутствует имя файла для отчета");
                }
                if (comboBoxDoctor.SelectedIndex < 0)
                {
                    throw new Exception("Не выбран доктор");
                }
                if (dateTimePickerDateEnd.Value <=
                dateTimePickerDateBegin.Value)
                {
                    throw new Exception("Дата начала должна быть раньше даты окончания");
                }
                if
                (_container.Resolve<TableReport>().CreateTable(textBoxFilePath.Text,
                (int)comboBoxDoctor.SelectedValue!,
                dateTimePickerDateBegin.Value, dateTimePickerDateEnd.Value))
                {
                    MessageBox.Show("Документ сформирован",
                    "Формирование документа",
                    MessageBoxButtons.OK,
                    MessageBoxIcon.Information);
                }
                else
                {
                    MessageBox.Show("Возникли ошибки при формировании документа.Подробности в логах",
                    "Формирование документа",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Ошибка при создании отчета",
                MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }
    }
}
