﻿using Microsoft.Extensions.Logging;
using ProjectPolyclinic.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPolyclinic.Reports;

internal class ChartReport
{
    private readonly IProfessionalDevelopmentRepository _professionalDevelopmentRepository;
    //private readonly IDoctorRepository _doctorRepository;
    private readonly ILogger<ChartReport> _logger;
    public ChartReport(IProfessionalDevelopmentRepository professionalDevelopmentRepository,
    IDoctorRepository doctorRepository,
    ILogger<ChartReport> logger)
    {
        _professionalDevelopmentRepository = professionalDevelopmentRepository ??
            throw new ArgumentNullException(nameof(professionalDevelopmentRepository));
        //_doctorRepository = doctorRepository ??
          //  throw new ArgumentNullException(nameof(doctorRepository));
        _logger = logger ??
            throw new ArgumentNullException(nameof(logger));
    }
    public bool CreateChart(string filePath, DateTime dateTime)
    {
        try
        {
            new PdfBuilder(filePath)
            .AddHeader("Повышение квалификации врачей")
            .AddPieChart("Количество повышений квалификации врачей", GetData(dateTime))
            .Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<(string Caption, double Value)> GetData(DateTime dateTime)
    {
        return _professionalDevelopmentRepository
        .ReadProfessionalDevelopment()
        .Where(x => x.ProfessionalDevelopmentTime.Date == dateTime.Date)
        .GroupBy(x => x.DoctorId, (key, group) => new { Id = key, Count = group.Count() })
        .Select(x => (x.Id.ToString(), (double)x.Count))
        .ToList();
    }
}
