﻿using Microsoft.Extensions.Logging;
using ProjectPolyclinic.Repositories;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPolyclinic.Reports;

internal class DocReport
{
    private readonly IDoctorRepository _doctorRepository;
    private readonly IPatientRepository _patientRepository;
    private readonly IMedicationRepository _medicationRepository;
    private readonly ILogger<DocReport> _logger;

    public DocReport(IDoctorRepository doctorRepository,IPatientRepository patientRepository,IMedicationRepository medicationRepository, ILogger<DocReport> logger)
    {
        _doctorRepository = doctorRepository ?? throw new ArgumentNullException(nameof(doctorRepository));
        _patientRepository = patientRepository ?? throw new ArgumentNullException(nameof(patientRepository));
        _medicationRepository = medicationRepository ?? throw new ArgumentNullException(nameof(medicationRepository));
        _logger = logger ?? throw new ArgumentNullException(nameof(logger));

    }
    public bool CreateDoc(string filePath, bool includeDoctors, bool includePatients, bool includeMedicines)
    {
        try
        {
            var builder = new WordBuilder(filePath)
            .AddHeader("Документ со справочниками");
            if (includeDoctors)
            {
                builder.AddParagraph("Врачи")
                .AddTable([2400, 2400, 2400],
                GetDoctors());
            }
            if (includePatients)
            {
                builder.AddParagraph("Пациенты")
                .AddTable([2400, 2400, 2400], GetPatients());
            }
            if (includeMedicines)
            {
                builder.AddParagraph("Медикаменты")
                .AddTable([2400, 2400, 2400], GetMedicines());
            }
            builder.Build();
            return true;
        }
        catch (Exception ex)
        {
            _logger.LogError(ex, "Ошибка при формировании документа");
            return false;
        }
    }
    private List<string[]> GetDoctors()
    {
        return [
            ["Имя доктора", "Фамилия доктора", "Специализация"],
            .. _doctorRepository
            .ReadDoctors()
            .Select(x => new string[] { x.First_Name, x.Last_Name, x.Specialization.ToString()}),
        ];
    }
    private List<string[]> GetPatients()
    {
        return [
            ["Имя пациента ", "Фамилия пациента", "Контактный номер"],
            .. _patientRepository
            .ReadPatients()
            .Select(x => new string[] { x.First_Name, x.Last_Name, x.ContactNumber.ToString()}),
        ];
    }
    private List<string[]> GetMedicines()
    {
        return [
            ["Тип таблетки ", "Доза", "Описание"],
            .. _medicationRepository
            .ReadMedicines()
            .Select(x => new string[] { x.MedicinesType.ToString(), x.Dosing, x.Description}),
        ];
    }
}
