﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPolyclinic.Entities;
using ProjectPolyclinic.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPolyclinic.Repositories.Implementations
{
    public class DoctorRepository:IDoctorRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<DoctorRepository> _logger;
        public DoctorRepository(IConnectionString connectionString, ILogger<DoctorRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreateDoctor(Doctor doctor)
        {
            _logger.LogInformation("Добавление объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(doctor));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryInsert = @"INSERT INTO Doctors (First_Name, Last_Name, Specialization) 
                                VALUES (@First_Name, @Last_Name, @Specialization)";
                connection.Execute(queryInsert, doctor);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void UpdateDoctor(Doctor doctor)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(doctor));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryUpdate = @"UPDATE Doctors SET 
                                           First_Name = @First_Name,
                                           Last_Name = @Last_Name,
                                           Specialization= @Specialization
                                           WHERE Id=@id";
                connection.Execute(queryUpdate, doctor);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void DeleteDoctor(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryDelete = @"DELETE FROM Doctors WHERE ID=@id";
                connection.Execute(queryDelete, new { id });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }
        public Doctor ReadDoctorById(int id)
        {
            _logger.LogInformation("Получение объекта по идентификатору");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @"SELECT * FROM Doctors WHERE Id=@id";
                var doctor = connection.QueryFirst<Doctor>(querySelect, new
                {
                    id
                });
                _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(doctor));
                return doctor;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }
        public IEnumerable<Doctor> ReadDoctors()
        {
            _logger.LogInformation("Получение всех объектов");
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM Doctors";
                var doctors = connection.Query<Doctor>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}",
                JsonConvert.SerializeObject(doctors));
                return doctors;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }
        
    }
}
