﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPolyclinic.Entities;
using ProjectPolyclinic.Entities.Enums;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPolyclinic.Repositories.Implementations
{
    public class MedicationRepository:IMedicationRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<MedicationRepository> _logger;
        public MedicationRepository(IConnectionString connectionString, ILogger<MedicationRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreateMedication(Medication medication)
        {
            _logger.LogInformation("Добавление объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(medication));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryInsert = @"INSERT INTO Medicines (MedicinesType, Dosing, Description) 
                                VALUES (@MedicinesType, @Dosing, @Description)";
                connection.Execute(queryInsert, medication);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void UpdateMedication(Medication medication)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(medication));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryUpdate = @"UPDATE Medicines SET 
                                           MedicinesType = @MedicinesType,
                                           Dosing = @Dosing,
                                           Description= @Description
                                           WHERE Id=@id";
                connection.Execute(queryUpdate, medication);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void DeleteMedication(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryDelete = @"DELETE FROM Medicines WHERE ID=@id";
                connection.Execute(queryDelete, new { id });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }
        public Medication ReadMedicationById(int id)
        {
            _logger.LogInformation("Получение объекта по идентификатору");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @"SELECT * FROM Medicines WHERE Id=@id";
                var medication = connection.QueryFirst<Medication>(querySelect, new
                {
                    id
                });
                _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(medication));
                return medication;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }
        public IEnumerable<Medication> ReadMedicines()
        {
            _logger.LogInformation("Получение всех объектов");
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM Medicines";
                var medicines = connection.Query<Medication>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}",
                JsonConvert.SerializeObject(medicines));
                return medicines;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }
        
    }
}
