﻿using Dapper;
using Microsoft.Extensions.Logging;
using Newtonsoft.Json;
using Npgsql;
using ProjectPolyclinic.Entities;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectPolyclinic.Repositories.Implementations
{
    public class PatientRepository:IPatientRepository
    {
        private readonly IConnectionString _connectionString;
        private readonly ILogger<PatientRepository> _logger;
        public PatientRepository(IConnectionString connectionString, ILogger<PatientRepository> logger)
        {
            _connectionString = connectionString;
            _logger = logger;
        }
        public void CreatePatient(Patient patient)
        {
            _logger.LogInformation("Добавление объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(patient));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryInsert = @"INSERT INTO Patients (First_Name, Last_Name, ContactNumber) 
                                VALUES (@First_Name, @Last_Name, @ContactNumber)";
                connection.Execute(queryInsert, patient);
            }
            catch(Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void UpdatePatient(Patient patient)
        {
            _logger.LogInformation("Редактирование объекта");
            _logger.LogDebug("Объект: {json}", JsonConvert.SerializeObject(patient));
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryUpdate = @"UPDATE Patients SET 
                                           First_Name = @First_Name,
                                           Last_Name = @Last_Name,
                                           ContactNumber = @ContactNumber
                                           WHERE Id=@id";
                connection.Execute(queryUpdate, patient);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при добавлении объекта");
                throw;
            }
        }
        public void DeletePatient(int id)
        {
            _logger.LogInformation("Удаление объекта");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var queryDelete = @"DELETE FROM Patients WHERE ID=@id";
                connection.Execute(queryDelete, new { id });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при удалении объекта");
                throw;
            }
        }
        public Patient ReadPatientById(int id)
        {
            _logger.LogInformation("Получение объекта по идентификатору");
            _logger.LogDebug("Объект: {id}", id);
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = @"SELECT * FROM Patients WHERE Id=@id";
                var patient = connection.QueryFirst<Patient>(querySelect, new
                {
                    id
                });
                _logger.LogDebug("Найденный объект: {json}",
                JsonConvert.SerializeObject(patient));
                return patient;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при поиске объекта");
                throw;
            }
        }
        public IEnumerable<Patient> ReadPatients()
        {
            _logger.LogInformation("Получение всех объектов");
            try
            {
                using var connection = new NpgsqlConnection(_connectionString.ConnectionString);
                var querySelect = "SELECT * FROM Patients";
                var patients = connection.Query<Patient>(querySelect);
                _logger.LogDebug("Полученные объекты: {json}",
                JsonConvert.SerializeObject(patients));
                return patients;
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка при чтении объектов");
                throw;
            }
        }
        
    }
}
